# encoding: utf-8

"""
qg_botsdk Library
~~~~~~~~~~~~~~~~~~~~~

一款简洁、容易上手，适用于QQ官方频道机器人的Python轻量级SDK；只需导入qg_botsdk.qg_bot.BOT即可快速使用

.. seealso::
    更多教程和相关资讯可参阅：
    https://thoughts.teambition.com/sharespace/6289c429eb27e90041a58b57/docs/6289c429eb27e90041a58b52

:copyright: (c) 2022 by GLGDLY.
:license: MIT, see LICENSE for more details.
"""

name = "qg_botsdk"
__all__ = ["qg_bot", "model"]
